#!/bin/bash
os=$(uname)
if [ "$os" = "Darwin" ]; then
    echo "macOS currently not supported." >&2; exit 1
fi

COMPOSE_BINARY=$(docker compose > /dev/null 2>&1 && echo "docker compose" || echo "docker-compose")

function help {
    cat << EOF
    $0 -w number_of_workers [-u allow_internet y/N] [-p engine_port] [-c certificate_file] [-k certificate_key_file] [-m mount_location] [-a admin_password]

   -a Admin password for the REST API, default 'admin'
   -m Mount url for the REST API, default '/'
   -p Port of the REST API, default '80'

   -u Allow worker containers to access the Internet, default 'n', to allow pass 'y'
      Due to security reasons, this is not allowed by default.
   -w Number of worker containers to spawn

   -c TLS certificate file path
   -k TLS certificate key file path
EOF
}

worker_internet="no"
admin_password="admin"

while getopts a:p:w:c:k:m:u: option
do
case "${option}"
in
p) engine_port=${OPTARG};;
w) worker_number=${OPTARG};;
c) crt_file="${OPTARG}";;
k) crt_key_file="${OPTARG}";;
m) mount_location="${OPTARG}";;
a) admin_password="${OPTARG}";;
u)  [[ ${OPTARG,,} == "y" ]] && worker_internet="yes";
    [[ ${OPTARG,,} == "n" ]] && worker_internet="no";
    [[ ${OPTARG,,} != "y" ]] && [[ ${OPTARG,,} != "n" ]] && { echo "Invalid value for -u '${OPTARG}'"; exit 1;};;
*) echo "Invalid option" && exit 1;;
esac
done

function rel_to_abs_path {
    [ -x "$(command -v realpath)" ] || {
        echo "$1 is a relative path and realpath utility was not found. Please provide either absolute path or install realpath"
        exit 1
    }
    realpath "$1"
}

if [ -z ${worker_number+x} ]; then
    help
    exit 1
fi;

re='^[0-9]+$'
if ! grep -q nginx docker-compose.yml; then
    if [ -n "$crt_file" ]; then
        echo "-crt option ignored as there is no nginx in your Engine configuration."
    fi;
    if [ -n "$crt_key_file" ]; then
        echo "-key option ignored as there is no nginx in your Engine configuration."
    fi;
elif [ -z "$crt_file" ]; then
    echo "WARNING: We strongly advise against using GAMS Engine without SSL encryption! Only use it in secure, local networks, because everything is sent unencrypted!" >&2;
    if [ -z ${engine_port+x} ]; then
        echo "WARNING: Engine port is not specified will use port 80"
        engine_port=80
    fi;
    if ! [[ $engine_port =~ $re ]] ; then
       echo "ERROR: Engine port is not a number" >&2; exit 1
    fi;

    sed -i -E "s/\.template-secure/\.template/g" docker-compose.yml
    sed -i -E "/443:443/d" docker-compose.yml

    sed -i -E "s/volumes: #for-ssl/#volumes: #for-ssl/" docker-compose.yml

    sed -i -E "s/  -.+cert.key\" #for-ssl/#  - \"\$crt-key-file:\/etc\/nginx\/certs\/cert.key\" #for-ssl/" docker-compose.yml
    sed -i -E "s/  -.+cert.crt\" #for-ssl/#  - \"\$crt-file:\/etc\/nginx\/certs\/cert.crt\" #for-ssl/" docker-compose.yml

    sed -i -E "s/##/#/" docker-compose.yml
else
    if [ -z "$crt_key_file" ]; then
        echo "No key file provided." >&2; exit 1
    fi;
    [[ "$crt_file" = /* ]] || {
        crt_file=$(rel_to_abs_path "$crt_file")
    }
    [[ "$crt_key_file" = /* ]] || {
        crt_key_file=$(rel_to_abs_path "$crt_key_file")
    }
    if [ ! -f "$crt_file" ]; then
        echo "Certificate file: ${crt_file} does not exist." >&2; exit 1
    fi;
    if [ ! -f "$crt_key_file" ]; then
        echo "Certificate key file: ${crt_key_file} does not exist." >&2; exit 1
    fi;

    if [ -n "${engine_port+x}" ] && [ "${engine_port}" != "80" ] && [ "${engine_port}" != "443" ]; then
        echo "INFO: Port settings were ignored. Engine will always listen on port 443 when running with SSL configuration!"
    fi;
    engine_port=80
    grep -q "\"443:443\"" docker-compose.yml || {
        sed -i -E "/[0-9]+:[0-9]+.*/a \ \ \ \ \ \ -\ \"443:443\"" docker-compose.yml
    }

    crt_file_escaped=$(printf '%s\n' "$crt_file" | sed -e 's/[\/&]/\\&/g')
    crt_key_file_escaped=$(printf '%s\n' "$crt_key_file" | sed -e 's/[\/&]/\\&/g')

    sed -i -E "s/[^ ]+:\/etc\/nginx\/certs\/cert\.key\"/\"$crt_key_file_escaped:\/etc\/nginx\/certs\/cert.key\"/" docker-compose.yml
    sed -i -E "s/[^ ]+:\/etc\/nginx\/certs\/cert\.crt\"/\"$crt_file_escaped:\/etc\/nginx\/certs\/cert.crt\"/" docker-compose.yml
    sed -i -E 's/#vol/vol/' docker-compose.yml
    sed -i -E 's/#  -/  -/' docker-compose.yml

    grep -q "\"443:443\"" docker-compose.yml || {
        sed -i -E "/[0-9]+:[0-9]+.*/a \ \ \ \ \ \ -\ \"443:443\"" docker-compose.yml
    }

    sed -i -E "s/\.template/\.template-secure/g" docker-compose.yml
    sed -i -E "s/\.template-secure-secure/\.template-secure/g" docker-compose.yml
fi

if ! [[ $worker_number =~ $re ]] ; then
   echo "ERROR: Worker number is not a number" >&2; exit 1
fi

if [ -n "$mount_location" ]; then
    if [[ "$mount_location" =~ "/" ]] ; then
        echo "ERROR: Mount location must not include forward slashes" >&2; exit 1
    fi
    mount_location_escaped=$(printf '%s\n' "$mount_location" | sed -e 's/[\/&]/\\&/g')
    mount_location_escaped_trailing="$mount_location_escaped\/"
    nginx_root_url=""
else
    nginx_root_url="\/engine"
fi

sed -i -E "s/UWSGI_MOUNT_LOCATION: .*/UWSGI_MOUNT_LOCATION: \/${mount_location_escaped_trailing}api/" docker-compose.yml
sed -i -E "s/mount_url: .*/mount_url: \"\/$mount_location_escaped\"/" docker-compose.yml
sed -i -E "s/mount_api_url: .*/mount_api_url: \"\/${mount_location_escaped_trailing}api\"/" docker-compose.yml
sed -i -E "s/root_url: .*/root_url: \"\/usr\/share\/nginx$nginx_root_url\"/" docker-compose.yml
sed -i -E "s/ENGINE_URL: .*/ENGINE_URL: \"\/${mount_location_escaped_trailing}api\"/" docker-compose.yml
sed -i -E "s/BASE_URL: .*/BASE_URL: \"\/$mount_location_escaped\"/" docker-compose.yml

if [ "$worker_internet" == "yes" ]; then
    sed -Ei '/ +worker:/{:j;/ +networks:/b end;n; T j;:end;n;s/$/\n      - worker-ext-network/;n;/worker-ext-network/d}' docker-compose.yml
    echo "Updated network settings of worker to allow Internet access"
else
    sed -Ei '/ +worker:/{:j;/ +networks:/b end;n; T j;:end;n;/worker-ext-network/d;n;/worker-ext-network/d}' docker-compose.yml
fi

sudo docker pull gams/engine-pre-migrate:latest || exit 1
sudo docker run -e "admin_password=${admin_password}" -v "$(pwd)":/home/mig_user/release -u "$(id -u):$(id -g)" --rm gams/engine-pre-migrate:latest || exit 1
sudo $COMPOSE_BINARY pull || exit 1
grep -q nginx docker-compose.yml && {
    sed -i -E "0,/[0-9]+:[0-9]+/{s//$engine_port:$engine_port/}" docker-compose.yml
    sed -i -E "s/ENGINE_HTTP_PORT: .*/ENGINE_HTTP_PORT: $engine_port/" docker-compose.yml
}
sudo $COMPOSE_BINARY up -d --scale worker="$worker_number"
